<?php

namespace Concrete\Package\AbExchange\Block\AbExchange;

use Concrete\Core\Block\BlockController;
use Core;
use Package;

/**
 * ab_exchange package for Concrete5
 * concrete5.org marketplace license
 * Copyright Copyright 2017-2018, Alex Borisov
 *
 * @author Alex Borisov <linuxoidoz@gmail.com>
 * @package Concrete\Package\ab_exchange
 */
 
class Controller extends BlockController
{
    
    protected $btWrapperClass = 'ccm-ui';
    protected $btInterfaceWidth = "600";
    protected $btInterfaceHeight = "400";
    protected $btTable = 'btAbExchange';
    
    public $ECB_rates = array();
    public $CBR_rates = array();
    public $ECB_updated = '';
    public $CBR_updated = '';
    public $ECB_status = '';
    public $CBR_status = '';
    public $status_latest = '';
    public $status_cached = '';
    
    public $cache_update = 3600; // time period to update exchange rates in cache
    
    public function getBlockTypeDescription() 
    {
        return t("Add International Currency Exchange block");
    }

    public function getBlockTypeName() 
    {
        return t("Currency Exchange");
    }

    public function on_start() 
    {
        $this->set('bank', $this->bank);
        $this->set('last_updated', t('Last updated: '));
        
        $this->status_latest = t('Updated');
        $this->status_cached = t('Cached');
        $server_error = t('Server error: unable to load exchange rates');
        $date_format = t('d.m.Y');
        
        // European Central Bank
        $ECB = t('European Central Bank');
        $ECB_descr = t('Exchange rates quoted against &euro;1.00');
        $this->set('ECB', $ECB);
        $this->set('ECB_descr', $ECB_descr);
        
        $ECB_data = $this->getECBxml();
        if (!empty($ECB_data)) {
            $ECB_XML = simplexml_load_string($ECB_data);
            $this->ECB_updated = date($date_format, strtotime($ECB_XML->Cube->Cube["time"]));
            $this->set('ECB_updated', $this->ECB_updated . ' (' . $this->ECB_status . ')');
            
            foreach($ECB_XML->Cube->Cube->Cube as $rate) {
                $this->ECB_rates[(string)$rate["currency"]] = $rate["rate"];
            }
            ksort($this->ECB_rates);
            $this->set('ECB_rates', $this->ECB_rates);
        }
        else {
            $this->set('server_error', $server_error);
        }
        
        // Central Bank of Russia
        $CBR = t('Central Bank of Russia');
        $CBR_descr = t('Exchange rates quoted in &#8381; (rubles)');
        $this->set('CBR', $CBR);
        $this->set('CBR_descr', $CBR_descr);
        
        $CBR_data = $this->getCBRxml();
        if (!empty($CBR_data)) {
            $CBR_XML = simplexml_load_string($CBR_data);
            $this->CBR_updated = date($date_format, strtotime($CBR_XML["Date"]));
            $this->set('CBR_updated', $this->CBR_updated . ' (' . $this->CBR_status . ')');
            
            foreach($CBR_XML->Valute as $rate) {
                $this->CBR_rates[(string)$rate->CharCode] = $rate->Value;
            }
            ksort($this->CBR_rates);
            $this->set('CBR_rates', $this->CBR_rates);
        }
        else {
            $this->set('server_error', $server_error);
        }
    }

    public function registerViewAssets($outputContent = '')
    {
        $this->requireAsset('javascript', 'jquery');
    }
    
    public function view() 
    {
        $this->set('currency_list', $this->currency_list);
    }

    public function save($args) 
    {
        $args['bank'] = isset($args['bank']) ? $args['bank'] : 'ECB';
        
        switch ($args['bank']) {
            case 'ECB':
                $args['currency_list'] = isset($args['ECB_currencies']) ? implode(',', $args['ECB_currencies']) : '';
                break;
            case 'CBR':
                $args['currency_list'] = isset($args['CBR_currencies']) ? implode(',', $args['CBR_currencies']) : '';
                break;
        }
        parent::save($args);
    }

    public function getECBxml() 
    {
        $expensiveCache = $this->app->make('cache/expensive');
        $cacheItem = $expensiveCache->getItem('AbExchange/ECB');
        if ($cacheItem->isMiss()) {
            $helperFile = $this->app->make('helper/file');
            @$XML = $helperFile->getContents("http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml");
            if($XML == FALSE) {
                return '';
            }
            $expensiveCache->save($cacheItem->set($XML)->expiresAfter($this->cache_update));
            $this->ECB_status = $this->status_latest;
        }
        else {
            $XML = $cacheItem->get();
            $this->ECB_status = $this->status_cached;
        }
        return $XML;
    }
    
    public function getCBRxml() 
    {
        $expensiveCache = $this->app->make('cache/expensive');
        $cacheItem = $expensiveCache->getItem('AbExchange/CBR');
        if ($cacheItem->isMiss()) {
            $helperFile = $this->app->make('helper/file');
            @$XML = $helperFile->getContents("http://www.cbr.ru/scripts/XML_daily.asp");
            if($XML == FALSE) {
                return '';
            }
            $expensiveCache->save($cacheItem->set($XML)->expiresAfter($this->cache_update));
            $this->CBR_status = $this->status_latest;
        }
        else {
            $XML = $cacheItem->get();
            $this->CBR_status = $this->status_cached;
        }
        return $XML;
    }
}
